@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.6.4.  */
+/* A Bison parser, made by GNU Bison 2.7.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.6.4"
+#define YYBISON_VERSION "2.7"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -69,8 +69,8 @@
 #define yynerrs         i386_nerrs
 
 /* Copy the first part of user declarations.  */
-/* Line 358 of yacc.c  */
-#line 1 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 371 of yacc.c  */
+#line 1 "/home/mark/src/elfutils/libcpu/i386_parse.y"
 
 /* Parser for i386 CPU description.
    Copyright (C) 2004, 2005, 2007, 2008, 2009 Red Hat, Inc.
@@ -286,7 +286,7 @@ extern FILE *outfile;
 static size_t best_mnemonic_bits;
 #endif
 
-/* Line 358 of yacc.c  */
+/* Line 371 of yacc.c  */
 #line 291 "i386_parse.c"
 
 # ifndef YY_NULL
@@ -352,8 +352,8 @@ extern int i386_debug;
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 374 of yacc.c  */
-#line 217 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 387 of yacc.c  */
+#line 217 "/home/mark/src/elfutils/libcpu/i386_parse.y"
 
   unsigned long int num;
   char *str;
@@ -364,7 +364,7 @@ typedef union YYSTYPE
   struct argument *arg;
 
 
-/* Line 374 of yacc.c  */
+/* Line 387 of yacc.c  */
 #line 369 "i386_parse.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
@@ -392,7 +392,7 @@ int i386_parse ();
 
 /* Copy the second part of user declarations.  */
 
-/* Line 377 of yacc.c  */
+/* Line 390 of yacc.c  */
 #line 397 "i386_parse.c"
 
 #ifdef short
@@ -858,47 +858,18 @@ do                                                              \
     }								\
 while (YYID (0))
 
-
+/* Error token number */
 #define YYTERROR	1
 #define YYERRCODE	256
 
-/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
-   If N is 0, then set CURRENT to the empty location which ends
-   the previous symbol: RHS[0] (always defined).  */
-
-#ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
-    do                                                                  \
-      if (YYID (N))                                                     \
-        {                                                               \
-          (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;        \
-          (Current).first_column = YYRHSLOC (Rhs, 1).first_column;      \
-          (Current).last_line    = YYRHSLOC (Rhs, N).last_line;         \
-          (Current).last_column  = YYRHSLOC (Rhs, N).last_column;       \
-        }                                                               \
-      else                                                              \
-        {                                                               \
-          (Current).first_line   = (Current).last_line   =              \
-            YYRHSLOC (Rhs, 0).last_line;                                \
-          (Current).first_column = (Current).last_column =              \
-            YYRHSLOC (Rhs, 0).last_column;                              \
-        }                                                               \
-    while (YYID (0))
-#endif
-
-#define YYRHSLOC(Rhs, K) ((Rhs)[K])
-
-
 
 /* This macro is provided for backward compatibility. */
-
 #ifndef YY_LOCATION_PRINT
 # define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
-
 #ifdef YYLEX_PARAM
 # define YYLEX yylex (YYLEX_PARAM)
 #else
@@ -961,7 +932,7 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
   switch (yytype)
     {
       default:
-	break;
+        break;
     }
 }
 
@@ -1203,7 +1174,6 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 {
   YYSIZE_T yysize0 = yytnamerr (YY_NULL, yytname[yytoken]);
   YYSIZE_T yysize = yysize0;
-  YYSIZE_T yysize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
   const char *yyformat = YY_NULL;
@@ -1266,11 +1236,13 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                     break;
                   }
                 yyarg[yycount++] = yytname[yyx];
-                yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
-                if (! (yysize <= yysize1
-                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-                  return 2;
-                yysize = yysize1;
+                {
+                  YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
+                  if (! (yysize <= yysize1
+                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                    return 2;
+                  yysize = yysize1;
+                }
               }
         }
     }
@@ -1290,10 +1262,12 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 # undef YYCASE_
     }
 
-  yysize1 = yysize + yystrlen (yyformat);
-  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-    return 2;
-  yysize = yysize1;
+  {
+    YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
+    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+      return 2;
+    yysize = yysize1;
+  }
 
   if (*yymsg_alloc < yysize)
     {
@@ -1353,7 +1327,7 @@ yydestruct (yymsg, yytype, yyvaluep)
     {
 
       default:
-	break;
+        break;
     }
 }
 
@@ -1364,16 +1338,16 @@ yydestruct (yymsg, yytype, yyvaluep)
 int yychar;
 
 
-#ifndef YYLVAL_INITIALIZE
-# define YYLVAL_INITIALIZE()
-#endif
 #ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
 # define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
 # define YY_IGNORE_MAYBE_UNINITIALIZED_END
 #endif
+#ifndef YY_INITIAL_VALUE
+# define YY_INITIAL_VALUE(Value) /* Nothing. */
+#endif
 
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE yylval;
+YYSTYPE yylval YY_INITIAL_VALUE(yyval_default);
 
 /* Number of syntax errors so far.  */
 int yynerrs;
@@ -1449,8 +1423,8 @@ yyparse ()
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yyss = yyssa;
-  yyvs = yyvsa;
+  yyssp = yyss = yyssa;
+  yyvsp = yyvs = yyvsa;
   yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1459,15 +1433,6 @@ yyparse ()
   yyerrstatus = 0;
   yynerrs = 0;
   yychar = YYEMPTY; /* Cause a token to be read.  */
-
-  /* Initialize stack pointers.
-     Waste one element of value and location stack
-     so that they stay on the same level as the state stack.
-     The wasted elements are never initialized.  */
-  yyssp = yyss;
-  yyvsp = yyvs;
-
-  YYLVAL_INITIALIZE ();
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1647,8 +1612,8 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-/* Line 1813 of yacc.c  */
-#line 248 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 248 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      if (error_message_count != 0)
 			error (EXIT_FAILURE, 0,
@@ -1659,26 +1624,26 @@ yyreduce:
     break;
 
   case 5:
-/* Line 1813 of yacc.c  */
-#line 262 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 262 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (3)].str), (yyvsp[(3) - (3)].num)); }
     break;
 
   case 6:
-/* Line 1813 of yacc.c  */
-#line 264 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 264 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (2)].str), -1); }
     break;
 
   case 7:
-/* Line 1813 of yacc.c  */
-#line 266 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 266 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (2)].str), -2); }
     break;
 
   case 8:
-/* Line 1813 of yacc.c  */
-#line 268 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 268 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      struct synonym *newp = xmalloc (sizeof (*newp));
 		      newp->from = (yyvsp[(2) - (3)].str);
@@ -1693,8 +1658,8 @@ yyreduce:
     break;
 
   case 12:
-/* Line 1813 of yacc.c  */
-#line 287 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 287 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      if ((yyvsp[(3) - (6)].field) != NULL && strcmp ((yyvsp[(3) - (6)].field)->name, "RE") != 0
 			  && strcmp ((yyvsp[(3) - (6)].field)->name, "R") != 0)
@@ -1780,8 +1745,8 @@ yyreduce:
     break;
 
   case 14:
-/* Line 1813 of yacc.c  */
-#line 373 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 373 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      struct known_bitfield search;
 		      search.name = (yyvsp[(1) - (1)].str);
@@ -1799,14 +1764,14 @@ yyreduce:
     break;
 
   case 15:
-/* Line 1813 of yacc.c  */
-#line 388 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 388 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { (yyval.field) = NULL; }
     break;
 
   case 16:
-/* Line 1813 of yacc.c  */
-#line 392 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 392 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      check_bits ((yyvsp[(3) - (3)].bit));
 
@@ -1819,8 +1784,8 @@ yyreduce:
     break;
 
   case 17:
-/* Line 1813 of yacc.c  */
-#line 402 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 402 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      check_bits ((yyvsp[(1) - (1)].bit));
 		      (yyval.bit) = (yyvsp[(1) - (1)].bit);
@@ -1828,8 +1793,8 @@ yyreduce:
     break;
 
   case 18:
-/* Line 1813 of yacc.c  */
-#line 409 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 409 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      struct bitvalue *runp = (yyvsp[(1) - (2)].bit);
 		      while (runp->next != NULL)
@@ -1840,14 +1805,14 @@ yyreduce:
     break;
 
   case 19:
-/* Line 1813 of yacc.c  */
-#line 417 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 417 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { (yyval.bit) = (yyvsp[(1) - (1)].bit); }
     break;
 
   case 20:
-/* Line 1813 of yacc.c  */
-#line 421 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 421 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1857,8 +1822,8 @@ yyreduce:
     break;
 
   case 21:
-/* Line 1813 of yacc.c  */
-#line 428 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 428 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1868,8 +1833,8 @@ yyreduce:
     break;
 
   case 22:
-/* Line 1813 of yacc.c  */
-#line 435 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 435 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      struct known_bitfield search;
@@ -1892,20 +1857,20 @@ yyreduce:
     break;
 
   case 23:
-/* Line 1813 of yacc.c  */
-#line 457 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 457 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { (yyval.arg) = (yyvsp[(2) - (2)].arg); }
     break;
 
   case 24:
-/* Line 1813 of yacc.c  */
-#line 459 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 459 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { (yyval.arg) = NULL; }
     break;
 
   case 25:
-/* Line 1813 of yacc.c  */
-#line 463 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 463 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      struct argument *runp = (yyvsp[(1) - (3)].arg);
 		      while (runp->next != NULL)
@@ -1918,8 +1883,8 @@ yyreduce:
     break;
 
   case 26:
-/* Line 1813 of yacc.c  */
-#line 473 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 473 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.arg) = xmalloc (sizeof (struct argument));
 		      (yyval.arg)->name = combine ((yyvsp[(1) - (1)].name));
@@ -1928,8 +1893,8 @@ yyreduce:
     break;
 
   case 27:
-/* Line 1813 of yacc.c  */
-#line 481 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 481 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      struct argname *runp = (yyvsp[(1) - (2)].name);
 		      while (runp->next != NULL)
@@ -1940,14 +1905,14 @@ yyreduce:
     break;
 
   case 28:
-/* Line 1813 of yacc.c  */
-#line 489 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 489 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     { (yyval.name) = (yyvsp[(1) - (1)].name); }
     break;
 
   case 29:
-/* Line 1813 of yacc.c  */
-#line 492 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 492 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = nfield;
@@ -1982,8 +1947,8 @@ yyreduce:
     break;
 
   case 30:
-/* Line 1813 of yacc.c  */
-#line 524 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 524 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1995,8 +1960,8 @@ yyreduce:
     break;
 
   case 31:
-/* Line 1813 of yacc.c  */
-#line 533 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 533 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -2006,8 +1971,8 @@ yyreduce:
     break;
 
   case 32:
-/* Line 1813 of yacc.c  */
-#line 540 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 1792 of yacc.c  */
+#line 540 "/home/mark/src/elfutils/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -2019,8 +1984,8 @@ yyreduce:
     break;
 
 
-/* Line 1813 of yacc.c  */
-#line 2024 "i386_parse.c"
+/* Line 1792 of yacc.c  */
+#line 1989 "i386_parse.c"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2251,8 +2216,8 @@ yyreturn:
 }
 
 
-/* Line 2076 of yacc.c  */
-#line 550 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
+/* Line 2055 of yacc.c  */
+#line 550 "/home/mark/src/elfutils/libcpu/i386_parse.y"
 
 
 static void
