@@ -252,6 +252,8 @@ execute_cfi (Dwarf_CFI *cache,
 	  continue;
 
 	case DW_CFA_expression:
+	  /* Expression rule relies on section data, abi_cfi cannot use it.  */
+	  assert (! abi_cfi);
 	  get_uleb128 (regno, program);
 	  offset = program - (const uint8_t *) cache->data->d.d_buf;
 	  /* DW_FORM_block is a ULEB128 length followed by that many bytes.  */
@@ -262,6 +264,8 @@ execute_cfi (Dwarf_CFI *cache,
 	  continue;
 
 	case DW_CFA_val_expression:
+	  /* Expression rule relies on section data, abi_cfi cannot use it.  */
+	  assert (! abi_cfi);
 	  get_uleb128 (regno, program);
 	  /* DW_FORM_block is a ULEB128 length followed by that many bytes.  */
 	  offset = program - (const uint8_t *) cache->data->d.d_buf;
