@@ -99,7 +99,7 @@ array_size (Dwarf_Die *die, Dwarf_Word *size,
 		    case DW_LANG_C89:
 		    case DW_LANG_C99:
 		    case DW_LANG_C_plus_plus:
-		    case DW_LANG_Objc:
+		    case DW_LANG_ObjC:
 		    case DW_LANG_ObjC_plus_plus:
 		    case DW_LANG_Java:
 		    case DW_LANG_D:
