@@ -1,28 +1,20 @@
 /* Print the strings of printable characters in files.
-   Copyright (C) 2005, 2006, 2007, 2009 Red Hat, Inc.
-   This file is part of Red Hat elfutils.
+   Copyright (C) 2005-2010, 2012 Red Hat, Inc.
+   This file is part of elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2005.
 
-   Red Hat elfutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 2 of the License.
+   This file is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
-   Red Hat elfutils is distributed in the hope that it will be useful, but
+   elfutils is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with Red Hat elfutils; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301 USA.
-
-   Red Hat elfutils is an included package of the Open Invention Network.
-   An included package of the Open Invention Network is a package for which
-   Open Invention Network licensees cross-license their patents.  No patent
-   license is granted, either expressly or impliedly, by designation as an
-   included package.  Should you wish to participate in the Open Invention
-   Network licensing program, please visit www.openinventionnetwork.com
-   <http://www.openinventionnetwork.com>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -169,9 +161,9 @@ main (int argc, char *argv[])
   if (remaining == argc)
     /* We read from standard input.  This we cannot do for a
        structured file.  */
-    result = read_fd (STDOUT_FILENO,
+    result = read_fd (STDIN_FILENO,
 		      print_file_name ? "{standard input}" : NULL,
-		      fstat64 (STDOUT_FILENO, &st) == 0
+		      (fstat64 (STDIN_FILENO, &st) == 0 && S_ISREG (st.st_mode))
 		      ? st.st_size : INT64_C (0x7fffffffffffffff));
   else
     do
@@ -228,7 +220,7 @@ print_version (FILE *stream, struct argp_state *state __attribute__ ((unused)))
 Copyright (C) %s Red Hat, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2009");
+"), "2012");
   fprintf (stream, gettext ("Written by %s.\n"), "Ulrich Drepper");
 }
 
@@ -548,17 +540,17 @@ read_block_no_mmap (int fd, const char *fname, off64_t from, off64_t fdlen)
       size_t nb = (size_t) n + ntrailer;
       if (nb >= min_len_bytes)
 	{
-	  /* We only use complete charactesr.  */
+	  /* We only use complete characters.  */
 	  nb &= ~(bytes_per_char - 1);
 
 	  process_chunk (fname, buf, from + nb, nb, &unprinted);
 
-	  /* If the last bytes of the buffer (module the character
+	  /* If the last bytes of the buffer (modulo the character
 	     size) have been printed we are not copying them.  */
 	  size_t to_keep = unprinted != NULL ? 0 : min_len_bytes;
 
-	  memmove (buf, buf + nb - to_keep, to_keep + nb);
-	  ntrailer = to_keep + nb;
+	  memmove (buf, buf + nb - to_keep, to_keep);
+	  ntrailer = to_keep;
 	  from += nb;
 	}
       else
@@ -578,8 +570,6 @@ read_block_no_mmap (int fd, const char *fname, off64_t from, off64_t fdlen)
 static int
 read_block (int fd, const char *fname, off64_t fdlen, off64_t from, off64_t to)
 {
-  assert ((off64_t) min_len_bytes < fdlen);
-
   if (elfmap == NULL)
     {
       /* We need a completely new mapping.  */
@@ -597,12 +587,14 @@ read_block (int fd, const char *fname, off64_t fdlen, off64_t from, off64_t to)
 	 read pointer.  */
       // XXX Eventually add flag which avoids this if the position
       // XXX is known to match.
-      if (lseek64 (fd, from, SEEK_SET) != from)
+      if (from != 0 && lseek64 (fd, from, SEEK_SET) != from)
 	error (EXIT_FAILURE, errno, gettext ("lseek64 failed"));
 
       return read_block_no_mmap (fd, fname, from, to - from);
     }
 
+  assert ((off64_t) min_len_bytes < fdlen);
+
   if (to < (off64_t) elfmap_off || from > (off64_t) (elfmap_off + elfmap_size))
     {
       /* The existing mapping cannot fit at all.  Map the new area.
